from requests import Response, PreparedRequest
from requests.status_codes import codes


class JsonToPythonHook:
    def __init__(self, json_data):
        self.__dict__ = json_data


class ResponseHandler:
    def __init__(self, response: Response):
        self._resp = response

    @property
    def resp(self) -> Response:
        return self._resp

    @property
    def req(self) -> PreparedRequest:
        return self._resp.request

    @property
    def body(self):
        return self._resp.json(object_hook=JsonToPythonHook)

    @property
    def status_code(self) -> int:
        return self._resp.status_code

    def is_array(self) -> bool:
        """
        Check if response body is in array instead of object
        :return: bool
        """
        return type(self.body) == list

    def is_success(self):
        """
        Check if status code is equal to 200
        :return: bool
        """
        return self._resp.status_code == codes.okay

    def is_internal_server_error(self):
        """
        Check if status code is equal to 500
        :return: bool
        """
        return self._resp.status_code == codes.internal_server_error



import logging

from api_nichotined import Api, BigQuery


class TestApi(Api):
    def __init__(self):
        super().__init__("https://restcountries.com")

    def get_upload(self):
        return self.get(path="/v3.1/name/indonesia")

    @classmethod
    def init_bigquery(cls):
        bq = BigQuery()

        bq.authenticate_client_with_json_cred_path("cred.json")
        res = bq.get_rows_from(query="""SELECT * FROM `dummy` LIMIT 1000""")
        assert type(res[0].column_a) == str


if __name__ == '__main__':
    logging.basicConfig(level=logging.INFO, format="%(asctime)s - %(levelname)s - %(message)s")

    response = TestApi().get_upload()
    assert response.is_array() is True
    assert response.is_success() is True

    # TestApi.init_bigquery()



import os

from google.cloud import bigquery
from google.cloud.bigquery import job


class BigQuery:
    def __init__(self):
        self._client = None

    @property
    def client(self) -> bigquery.Client:
        return self._client

    @client.setter
    def client(self, new_client: bigquery.Client):
        self._client = new_client

    @client.deleter
    def client(self):
        del self._client

    def authenticate_client_with_json_cred_path(self, json_cred_path: str):
        """
        Initialize BigQuery client
        :param json_cred_path: path of credential file
        """
        if os.path.exists(json_cred_path):
            self.client = bigquery.Client.from_service_account_json(json_credentials_path=json_cred_path)
        else:
            raise FileNotFoundError("JSON file cannot be located")

    def run_query(self, query) -> job.QueryJob:
        """
        Run a SQL query and return QueryJob.
        :param query: query in string
        :return: Query Job object
        """
        return self.client.query(query)

    def get_rows_from(self, query):
        """
        Run a SQL query and return list of rows
        :param query: query in string
        :return: A List of rows
        """
        rows = self.run_query(query)
        result = []
        for row in rows:
            result.append(row)
        return result
